from zap_flask_pubsub.brokers.rabbit_mq import RabbitMQBroker


class BrokerFactory:
    @staticmethod
    def get_broker(app):
        if app.config.get('RABBITMQ_USER') and app.config.get('RABBITMQ_PWD'):
            return RabbitMQBroker(app)
        else:
            raise Exception('Broker is not configured')



import json
from zap_flask_pubsub.factory import BrokerFactory


class Subscriber:
    def __init__(self, app):
        self.app = app
        self.broker = BrokerFactory.get_broker(app)

    def setup(self):
        queues = json.load(open(self.app.config.get('PUBSUB_SUBSCRIPTION_LIST')))
        self.broker.run(queues)



import importlib
import threading
from abc import ABCMeta, abstractmethod


class Broker():
    __metaclass__ = ABCMeta

    def __init__(self, app):
        self.app = app
        try:
            self.callback_functions = importlib.import_module(app.config.get('PUBSUB_SUBSCRIPTION_CALLBACK'))
        except Exception as e:
            raise Exception('Please define PUBSUB_SUBSCRIPTION_CALLBACK in settings file')

    @classmethod
    @abstractmethod
    def get_connection(cls):
        """Return Broker connections"""

    @abstractmethod
    def run(self, queues):
        """Consuming message stream"""

    @abstractmethod
    def publish(self, data, routing_key, exchange=''):
        """publish the data in message queue"""


def validate_queues(queues):
    if type(queues) != list:
        raise Exception('queues must be a list')



import json
import pika

from zap_flask_pubsub.brokers.broker import Broker, validate_queues


class RabbitMQBroker(Broker):
    @classmethod
    def get_connection(cls, app):
        credentials = pika.PlainCredentials(app.config.get('RABBITMQ_USER'),
                                            app.config.get('RABBITMQ_PWD'))
        return pika.BlockingConnection(pika.ConnectionParameters(
            app.config.get('RABBITMQ_HOST', 'localhost'),
            app.config.get('RABBITMQ_PORT'), '/',
            credentials))

    def publish(self, data, routing_key, exchange=''):
        connection = RabbitMQBroker.get_connection(self.app)
        channel = connection.channel()

        channel.queue_declare(queue=routing_key)
        channel.basic_publish(exchange=exchange,
                              routing_key=routing_key,
                              body=json.dumps(data))
        connection.close()

    def run(self, queues):
        '''
        :param queues:[{id:<queue_id>}, callback:<callback_function>]
        :return: None
        '''
        validate_queues(queues)
        connection = RabbitMQBroker.get_connection(self.app)
        channel = connection.channel()
        for queue in queues:
            channel.queue_declare(queue=queue.get('routing_key'))
            #  channel.basic_consume(queue=queue.get('id'), on_message_callback=queue.get('callback'))
            callback = getattr(self.callback_functions, queue.get('callback'))
            channel.basic_consume(queue=queue.get('routing_key'), on_message_callback=callback)
        print('Subscriber is listening...')
        channel.start_consuming()



from abc import ABCMeta

from zap_flask_pubsub.factory import BrokerFactory


class Publisher:
    __metaclass__ = ABCMeta

    def __init__(self, app):
        self.broker = BrokerFactory.get_broker(app)

    def publish(self, data, routing_key, exchange=''):
        self.broker.publish(data, routing_key, exchange)
